<?php

class FlyerTech_Transactionale_Helper_Data extends Mage_Core_Helper_Abstract
{
    public function isPublisher()
    {
        $this->ovverideLegacyEnabledConfig();
        $mode = Mage::getStoreConfig('transactionale/general/integration_mode', Mage::app()->getStore());
        switch ($mode) {
            case 'advertiser':
                return false;
            case 'disabled':
                return false;
        }

        return true;
    }

    public function isEnabled()
    {
        $this->ovverideLegacyEnabledConfig();
        $mode = Mage::getStoreConfig('transactionale/general/integration_mode', Mage::app()->getStore());
        switch ($mode) {
            case 'disabled':
                return false;
        }

        return true;
    }

    public function isTrackTransactionClientIntegration()
    {
        $this->ovverideLegacyEnabledConfig();

        return Mage::getStoreConfig('transactionale/general/integration_sending_mode',
                Mage::app()->getStore()) == 'client';
    }

    private function ovverideLegacyEnabledConfig()
    {
        $leagacyConfig = Mage::getStoreConfig('transactionale/general', Mage::app()->getStore());
        if (isset($leagacyConfig['enabled']) && !isset($leagacyConfig['integration_mode'])) {
            if ($leagacyConfig['enabled']) {
                Mage::getConfig()->saveConfig('transactionale/general/integration_mode', 'both', 'default', 0);
            } else {
                Mage::getConfig()->saveConfig('transactionale/general/integration_mode', 'disabled', 'default', 0);
            }
        }
        if (!isset($leagacyConfig['integration_sending_mode'])) {
            Mage::getConfig()->saveConfig('transactionale/general/integration_sending_mode', 'client', 'default', 0);
        }
    }

    private function convertPayloadInTrxCall($payload)
    {
        $moduleVersion = "";
        $magentoVersion = "";

        $versionInfo = Mage::getVersionInfo();
        if (!empty($versionInfo)) {
            $magentoVersion = "{$versionInfo['major']}.{$versionInfo['minor']}.{$versionInfo['revision']}.{$versionInfo['patch']}";
        }
        if (!empty(Mage::getConfig()->getNode()->modules->FlyerTech_Transactionale->version[0])) {
            $moduleVersion = Mage::getConfig()->getNode()->modules->FlyerTech_Transactionale->version[0] . "";
        }

        $apikey = Mage::getStoreConfig('transactionale/general/api_key', Mage::app()->getStore());
        $country = Mage::getStoreConfig('transactionale/general/country_code', Mage::app()->getStore());
        $trx = [];
        $trx['uk'] = $apikey;
        $trx['vv'] = '1.0.0';
        $trx['uc'] = $country;
        $trx['fs'] = 'false';

        $trx['pn'] = 'magento';
        $trx['pv'] = $magentoVersion;
        $trx['pi'] = $moduleVersion;

        if (isset($payload['customer'])) {
            if (isset($payload['customer']['email'])) {
                $payload['email'] = $payload['customer']['email'];
                unset($payload['customer']['email']);
            }
        }

        $mapping = $this->getPayloadKeysMapping();
        foreach ($payload as $key => $value) {
            //$this->log($key . "(" . $mapping[$key] . ") " . json_encode($subValue));

            if ($value === false || $value === '') {
                continue;
            } elseif ($key == "items") {
                continue;
            } elseif ($key == "itemIDs") {
                continue;
            } elseif (is_array($value)) {
                foreach ($value as $subKey => $subValue) {
                    //$this->log("\t|- " . $subKey . "(" . $mapping[$key][$subKey] . ") " . json_encode($subValue));
                    if ($value === false || $value === '') {
                        continue;
                    }
                    $trx[$mapping[$key][$subKey]] = $subValue;
                }
            } else {
                $trx[$mapping[$key]] = $value;
            }
        }

        if (isset($trx['ue'])) {
            $trx['ue'] = base64_encode($trx['ue']);
        }

        if (isset($payload['items'])) {
            $k = 1;
            foreach ($payload['items'] as $item) {
                $key = 'items';
                //$this->log($k . " " . json_encode($item));
                foreach ($item as $subKey => $subValue) {
                    if ($subValue === false || $subValue === '') {
                        continue;
                    }
                    //$this->log("\t|- " . $subKey . "-> " . ($mapping[$key][$subKey] . $k) . " " . json_encode($subValue));
                    $trx[$mapping[$key][$subKey] . $k] = $subValue;
                }
                $k++;
            }
        }
        if (isset($payload['itemIDs'])) {
            $k = 1;
            foreach ($payload['itemIDs'] as $item) {
                $trx[$mapping['itemIDs'] . $k] = $item;
                //$this->log($key . " " . json_encode($item));

                $k++;
            }
        }
        return $trx;
    }

    private function getPayloadKeysMapping()
    {
        return [
            'auth'               => 'uk',
            'country'            => 'uc',
            'version'            => 'vv',
            'email'              => 'ue',
            'event'              => 'ev',
            'touchpointId'       => 'ut',
            'touchpointType'     => 'uy',
            'isTransaConversion' => 'ec',
            'customEventName'    => 'en',
            'bookingId'          => 'bi',
            'bookingHash'        => 'bh',
            'userHash'           => 'uh',

            'forceSend'   => 'fs',
            'platform'    => [
                'name'               => 'pn',
                'version'            => 'pv',
                'integrationVersion' => 'pi',
            ],
            'user'        => [
                'res'  => 'ur',
                'ua'   => 'ua',
                'lang' => 'ul'
            ],
            'page'        => [
                'href'        => 'ph',
                'lang'        => 'pl',
                'title'       => 'pt',
                'charset'     => 'pc',
                'keywords'    => 'pk',
                'description' => 'pd'
            ],
            'transaction' => [
                'id'       => 'ti',
                'subtotal' => 'ts',
                'shipping' => 'th',
                'discount' => 'td',
                'coupon'   => 'tc',
                'total'    => 'tt',
                'currency' => 'tv'
            ],
            'itemIDs'     => 'ji', // No sub-fields, so one-level key only
            'items'       => [
                'id'           => 'ii',
                'price'        => 'ip',
                'quantity'     => 'iq',
                'name'         => 'in',
                'categoryId'   => 'ic',
                'categoryName' => 'id',
                'barcode'      => 'ib'
            ],  // EAN or UPC],
            'address'     => [
                'firstName'                    => 'af',
                'lastName'                     => 'al',
                'company'                      => 'ac',
                'address'                      => 'aa',
                'address2'                     => 'ab',
                'city'                         => 'at',
                'postalCode'                   => 'az',
                'country'                      => 'an',
                'phone'                        => 'ap',
                'phoneMobile'                  => 'am',
                'nationalIdentificationNumber' => 'ni'
            ],
            'customer'    => [
                'firstName' => 'cf',
                'lastName'  => 'cl',
                'company'   => 'cc',
                'birthDate' => 'cb',
                'birthdate' => 'cb', // Support for a mistake in the docs
                'gender'    => 'cg',
                'optin'     => 'co'
            ]
        ];
    }

    public function getTrackTransaction($order_id)
    {
        $order = Mage::getModel('sales/order')->load($order_id);
        $touchpointType = Mage::getStoreConfig('transactionale/general/touchpoint_type',
            Mage::app()->getStore());
        $touchpointId = Mage::getStoreConfig('transactionale/general/touchpoint_id', Mage::app()->getStore());
        $countryId = Mage::getStoreConfig('transactionale/general/country_code', Mage::app()->getStore());

        $shippingAddress = $order->getShippingAddress();
        $billingAddress = $order->getBillingAddress();
        $customer_id = $order->getCustomerId();
        $customerGender = '';
        $customerDob = '';
        if ($customer_id) {
            $customer = Mage::getModel('customer/customer')->load($customer_id);
            $customerGender = $customer->getGender();
            $customerDob = $customer->getDob();
        }
        if ($customerGender == 1) {
            $customerGender = 'm';
        } elseif ($customerGender == 2) {
            $customerGender = 'f';
        } else {
            $customerGender = '';
        }
        $address = str_replace(["\n", "\r"], " ", $shippingAddress->getStreetFull());
        $address = htmlspecialchars($address);
        $items = [];
        foreach ($order->getAllItems() as $_product) {
            $categories = [];
            $cats = $_product->getProduct()->getCategoryIds();
            foreach ($cats as $categoryId) {
                $cat = Mage::getModel('catalog/category')->setStoreId(Mage::app()->getStore()->getId())->load($categoryId);
                $categories[] = $cat->getName();
            }
            $categories = implode(",", $categories);
            $items[] = [
                'id'           => $_product->getSku(),
                'quantity'     => intval($_product->getQtyOrdered()),
                'price'        => number_format($_product->getPrice(), 2, '.', ''),
                'name'         => $_product->getName(),
                'categoryName' => $categories
            ];
        }

        $transaction = [
            'event'          => 'trackTransaction',
            'transaction'    => [
                'id'       => $order->getIncrementId(),
                'subtotal' => number_format($order->getSubtotal(), 2, '.', ''),
                'shipping' => number_format($order->getShippingAmount(), 2, '.', ''),
                'discount' =>
                    number_format($order->getDiscountAmount() < 0 ? $order->getDiscountAmount() * (-1) : $order->getDiscountAmount(),
                        2, '.', ''),
                'total'    => number_format($order->getGrandTotal(), 2, ' . ', ''),
                'coupon'   => $order->getCouponCode(),
                'currency' => $order->getOrderCurrencyCode(),
            ],
            'items'          => $items,
            'address'        => [
                'address'     => $address,
                'city'        => $shippingAddress->getCity(),
                'postalCode'  => $shippingAddress->getPostcode(),
                'country'     => $shippingAddress->getCountry_id(),
                'phone'       => $shippingAddress->getTelephone(),
                'phoneMobile' => $shippingAddress->getTelephone(),
            ],
            'customer'       => [
                'firstName' => $order->getCustomerFirstname(),
                'lastName'  => $order->getCustomerLastname(),
                'company'   => $billingAddress->getCompany(),
                'email'     => $order->getCustomerEmail(),
                'optin'     => $order->getCustomerOptIn() ,
                'birthdate' => $customerDob,
                'gender'    => $customerGender,
            ],
            'touchpointType' => $touchpointType,
        ];


        if ($touchpointType != "mail") {
            if ($touchpointId) {
                $transaction['touchpointId'] = $touchpointId;
            }
            $transaction['containerId'] = 'transactionale_touchpoint_div';
        }

        return $transaction;
    }

    public function trackOrder($order_id)
    {
        $transaction = $this->getTrackTransaction($order_id);
        // Conversione

        $trx = $this->convertPayloadInTrxCall($transaction);
        $this->log(json_encode($trx));

        $url = "https://trx.transactionale.com/e";

        $final = $url . "?" . http_build_query($trx);
        $ch = curl_init($final);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

        $response = curl_exec($ch);
        curl_close($ch);
    }

    public function log($value)
    {
        if ($value instanceof Exception) {
            Mage::log($value->getMessage());
            Mage::log($value->getTraceAsString());
        } else {
            Mage::log($value, null, "transactionale.log");
        }
    }
}